@@ -30,9 +30,6 @@ use tokio::sync::Mutex;
 
 use crate::{internal_error_map, error::*};
 
-lazy_static!{
-    pub(super) static ref ROUND_ROBIN_CNT: Mutex<usize> = Mutex::new(0);
-}
 
 #[async_trait]
 pub trait ConfigParser<T>
