@@ -533,7 +533,7 @@ impl<'req> QDns<'req>
 
        // let mut resolved: HashSet<&'req QDnsReq> = HashSet::with_capacity(self.ordered_req_list.len());
 
-        for resolver in self.resolvers.get_resolvers_iter()?
+        for resolver in self.resolvers.get_resolvers_iter()
         {
             let qresp = 
                 self.processing(now, resolver, None, false).await?;
