@@ -24,6 +24,7 @@
 use std::hash::{Hash};
 use std::net::{IpAddr, SocketAddr};
 use std::str::FromStr;
+use std::sync::atomic::{AtomicUsize, Ordering};
 
 use bitflags::bitflags;
 
@@ -33,6 +34,65 @@ use crate::tokenizer::*;
 
 use super::common::{IPV4_BIND_ALL, IPV6_BIND_ALL};
 
+lazy_static!{
+    /// A rounrobin counter. It was intentionally designed to count forward as
+    /// it is not expected that the counter will ever be overflowed.
+    pub(super) static ref ROUND_ROBIN_CNT: AtomicUsize = AtomicUsize::new(0);
+}
+
+
+/// An implementaion of RoundRobin iterator. This iterator is sharing a single
+/// source of counting to implement roundrobin functionality.
+pub struct RoundRobinIterator<'iterator>
+{
+    end_index: usize,
+    current_index: usize,
+    slice: &'iterator [ResolveConfEntry]
+}
+
+impl<'iterator> RoundRobinIterator<'iterator>
+{
+    pub 
+    fn new(start_index: usize, slice: &'iterator [ResolveConfEntry]) -> RoundRobinIterator<'iterator>
+    {        
+        return 
+            RoundRobinIterator
+            {
+                end_index: start_index + slice.len(),
+                current_index: start_index,
+                slice: slice
+            };
+    }
+
+    pub 
+    fn len(&self) -> usize
+    {
+        return self.slice.len();
+    }
+} 
+
+impl<'iterator> Iterator for RoundRobinIterator<'iterator>
+{
+    type Item = &'iterator ResolveConfEntry;
+
+    fn next(&mut self) -> Option<Self::Item> 
+    {
+        let temp_index = self.current_index;
+        
+        self.current_index += 1;
+
+        if temp_index == self.end_index
+        {
+            return None;
+        }
+
+        // compute real index
+        let real_index = temp_index % self.slice.len();
+
+        return self.slice.get(real_index);
+    }
+}
+
 /// A resolve.conf parser
 #[derive(Clone, Debug)]
 pub struct ResolveConfEntry 
@@ -377,56 +437,6 @@ impl OptionFlags
     }
 }
 
-pub struct RoundRobinIterator<'iterator>
-{
-    end_cycle: usize,
-    current_index: usize,
-    slice: &'iterator [ResolveConfEntry]
-}
-
-impl<'iterator> RoundRobinIterator<'iterator>
-{
-    pub 
-    fn new(start_index: usize, slice: &'iterator [ResolveConfEntry]) -> RoundRobinIterator<'iterator>
-    {
-        if start_index == slice.len()
-        {
-            panic!("Assertion trap: start_index == slice.len i.e {} == {}", start_index, slice.len());
-        }
-        
-        return 
-            RoundRobinIterator
-            {
-                end_cycle: start_index + slice.len(),
-                current_index: start_index,
-                slice: slice
-            };
-    }
-} 
-
-impl<'iterator> Iterator for RoundRobinIterator<'iterator>
-{
-    type Item = &'iterator ResolveConfEntry;
-
-    fn next(&mut self) -> Option<Self::Item> 
-    {
-        let mut curr = self.current_index;
-        
-        self.current_index += 1;
-
-        if /*self.current_index*/curr == self.end_cycle
-        {
-            return None;
-        }
-
-        if curr >= self.slice.len()
-        {
-            curr -= self.slice.len();
-        }
-
-        return self.slice.get(curr);
-    }
-}
 
 #[derive(Clone, Debug, Default)]
 pub struct ResolveConfig
@@ -445,6 +455,29 @@ pub struct ResolveConfig
 
 impl ResolveConfig
 {
+    /// This function will either return the [RoundRobinIterator] which always 
+    /// starts from zero offset OR it will return the same instace but the initial
+    /// offset will be [ROUND_ROBIN_CNT] shared atomic counter. The [ROUND_ROBIN_CNT]
+    /// counts always forward. It is not expected that it will ever be overflowed.
+    /// 
+    /// # Returns
+    /// 
+    /// * [RoundRobinIterator] instance
+    pub 
+    fn get_resolvers_iter(&self) -> RoundRobinIterator 
+    {
+        if self.option_flags.is_rotate() == true
+        {
+            let start_index = ROUND_ROBIN_CNT.fetch_add(1, Ordering::SeqCst);
+
+            return RoundRobinIterator::new(start_index, self.nameservers.as_slice());
+        }
+        else
+        {
+            return RoundRobinIterator::new(0, self.nameservers.as_slice());
+        };
+    }
+
     unsafe
     fn split_dns_ifr(field_data: &str, ifr: Option<&IfInfo>) -> CDnsResult<(IpAddr, SocketAddr)>
     {
@@ -538,18 +571,11 @@ impl ResolveConfig
         }
     }
 
-    //https://man.openbsd.org/OpenBSD-6.3/resolv.conf.5
-    //https://mail-index.netbsd.org/port-arm32/1998/10/25/0006.html
-    //https://man7.org/linux/man-pages/man5/resolv.conf.5.html
-    //https://serverfault.com/questions/732325/how-to-persist-resolv-conf-options-such-as-rotate-timeout-in-centos
-    //https://www.freebsd.org/cgi/man.cgi?resolv.conf
-    //https://www.shellhacks.com/setup-dns-resolution-resolvconf-example/
     /// Parses the /etc/resolv.conf
     /// Different OSes like GNU/Linux, FreeBSD and OpenBSD uses different 
     /// configuration options. As this lib is licensed as LGPL, it is not really
     /// the BSD way. The main implementation is like it is described in 
     /// resolv.conf(5) of Linux man.
-    /// Todo: add conditional compilation to compile for different targets.
     pub(crate)
     fn parser_resolv_internal(file_content: &str, f: &mut Writer) -> CDnsResult<Self>
     {
@@ -1097,15 +1123,116 @@ search   localdomain\n\
     assert_eq!(res.option_flags.contains(OptionFlags::OPT_SINGLE_REQUEST_REOPEN), true);
 }
 
+#[test]
+fn test_roundrobin_iter_1() 
+{
+    use std::time::Instant;
+
+    fn check_order(res: &ResolveConfig, ipl: &[IpAddr], indx: usize)
+    {
+        let mut i = indx;
+        let assert_i = (indx + ipl.len()) % ipl.len();
+        let now = Instant::now();
+
+        let itr = res.get_resolvers_iter();
+
+        assert_eq!(itr.len(), ipl.len());
+
+        for n in itr
+        {
+            //println!("cmp: {} {}", n.ip, ipl[i]);
+            assert_eq!(n.ip, ipl[i]);
+
+            i += 1;
+
+            i %= ipl.len();
+
+        }
+
+        assert_eq!(i, assert_i);
+
+        let elapsed = now.elapsed();
+        println!("Iter Elapsed: {:.2?}", elapsed);
+    }
+
+    let ip1: IpAddr = "192.168.2.1".parse().unwrap();
+
+    let ip_list: Vec<IpAddr> = vec![ip1];
+
+    let test = 
+"\
+# test 12345\n\
+  # nameserver    192.168.3.1\n\
+nameserver  192.168.2.1  \n\
+#nameserver       127.0.0.1   \n\
+lookup file\n\
+family inet4       \n\
+#nameserver  8.8.8.8\n\
+options    attempts:1  rotate   timeout:3 debug use-vc     single-request-reopen\n\
+search   localdomain\n\
+";
+ 
+let now = Instant::now();
+
+    let mut writer = Writer::new();
+
+    let res = ResolveConfig::parser_resolv_internal(test, &mut writer);
+
+let elapsed = now.elapsed();
+println!("parser_resolv_internal elapsed: {:.2?}", elapsed);
+
+    assert_eq!(res.is_ok(), true, "{}", res.err().unwrap());
+    let res = res.unwrap();
+
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 0);
+}
+
 #[test]
 fn test_roundrobin_iter() 
 {
     use std::time::Instant;
 
+    fn check_order(res: &ResolveConfig, ipl: &[IpAddr], indx: usize)
+    {
+        let mut i = indx;
+        let assert_i = (indx + ipl.len()) % ipl.len();
+        let now = Instant::now();
+
+        let itr = res.get_resolvers_iter();
+
+        assert_eq!(itr.len(), ipl.len());
+
+        for n in itr
+        {
+            //println!("cmp: {} {}", n.ip, ipl[i]);
+            assert_eq!(n.ip, ipl[i]);
+
+            i += 1;
+
+            i %= ipl.len();
+
+        }
+
+        assert_eq!(i, assert_i);
+
+        let elapsed = now.elapsed();
+        println!("Iter Elapsed: {:.2?}", elapsed);
+    }
+
     let ip1: IpAddr = "192.168.2.1".parse().unwrap();
     let ip2: IpAddr = "127.0.0.1".parse().unwrap();
     let ip3: IpAddr = "8.8.8.8".parse().unwrap();
 
+    let ip_list: Vec<IpAddr> = vec![ip1, ip2, ip3];
+
     let test = 
 "\
 # test 12345\n\
@@ -1126,48 +1253,20 @@ let now = Instant::now();
     let res = ResolveConfig::parser_resolv_internal(test, &mut writer);
 
 let elapsed = now.elapsed();
-println!("Elapsed: {:.2?}", elapsed);
+println!("parser_resolv_internal elapsed: {:.2?}", elapsed);
 
     assert_eq!(res.is_ok(), true, "{}", res.err().unwrap());
     let res = res.unwrap();
 
-    let mut i: usize = 0;
-    
-    let now = Instant::now();
-
-    for n in res.get_resolvers_iter().unwrap()
-    {
-        match i
-        {
-            0 => { assert_eq!(n.ip, ip1); i += 1; },
-            1 => { assert_eq!(n.ip, ip2); i += 1; },
-            2 => { assert_eq!(n.ip, ip3); i += 1; },
-            _ => panic!("!")
-        }
-
-    }
-
-    assert_eq!(i, 3);
-
-    let elapsed = now.elapsed();
-    println!("Elapsed: {:.2?}", elapsed);
-
-    i = 0;
-    for n in res.get_resolvers_iter().unwrap()
-    {
-        match i
-        {
-            0 => { assert_eq!(n.ip, ip2); i += 1; },
-            1 => { assert_eq!(n.ip, ip3); i += 1; },
-            2 => { assert_eq!(n.ip, ip1); i += 1; },
-            _ => panic!("!")
-        }
-
-
-    }
-
-    assert_eq!(i, 3);
-
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 1);
+    check_order(&res, &ip_list, 2);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 1);
+    check_order(&res, &ip_list, 2);
+    check_order(&res, &ip_list, 0);
+    check_order(&res, &ip_list, 1);
+    check_order(&res, &ip_list, 2);
 }
 
 #[test]
@@ -1176,12 +1275,14 @@ fn test_direct_iter()
     use std::time::Instant;
 
     let ip1: IpAddr = "192.168.2.1".parse().unwrap();
+    let ip2: IpAddr = "192.168.2.2".parse().unwrap();
 
     let test = 
 "\
 # test 12345\n\
   # nameserver    192.168.3.1\n\
 nameserver  192.168.2.1  \n\
+nameserver  192.168.2.2  \n\
 lookup file\n\
 family inet4       \n\
 options    attempts:1  timeout:3 debug use-vc     single-request-reopen\n\
@@ -1204,31 +1305,33 @@ println!("Elapsed: {:.2?}", elapsed);
     
     let now = Instant::now();
 
-    for n in res.get_resolvers_iter().unwrap()
+    for n in res.get_resolvers_iter()
     {
         match i
         {
             0 => { assert_eq!(n.ip, ip1); i += 1; },
+            1 => { assert_eq!(n.ip, ip2); i += 1; },
             _ => panic!("!")
         }
     }
 
     let elapsed = now.elapsed();
-    println!("Elapsed: {:.2?}", elapsed);
+    println!("Iter Elapsed: {:.2?}", elapsed);
 
-    assert_eq!(i, 1);
+    assert_eq!(i, 2);
 
     i = 0;
-    for n in res.get_resolvers_iter().unwrap()
+    for n in res.get_resolvers_iter()
     {
         match i
         {
             0 => { assert_eq!(n.ip, ip1); i += 1; },
+            1 => { assert_eq!(n.ip, ip2); i += 1; },
             _ => panic!("!")
         }
     }
 
-    assert_eq!(i, 1);
+    assert_eq!(i, 2);
 }
 
 
