@@ -18,9 +18,10 @@
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
+
 /// This file contains the config file parsers.
 
-use std::io::Read;
+use std::{io::Read, sync::atomic::AtomicUsize};
 use std::fs::File;
 
 use std::path::Path;
@@ -28,12 +29,6 @@ use std::path::Path;
 use crate::{internal_error_map, error::*};
 
 
-use super::mutex::Mutex;
-
-lazy_static!{
-    pub(super) static ref ROUND_ROBIN_CNT: Mutex<usize> = Mutex::new(0);
-}
-
 pub trait ConfigParser<T>
 {
     fn parse_config() -> CDnsResult<T>;
