@@ -516,7 +516,7 @@ impl<'req> QDns<'req>
 
        // let mut resolved: HashSet<&'req QDnsReq> = HashSet::with_capacity(self.ordered_req_list.len());
 
-        for resolver in self.resolvers.get_resolvers_iter()?
+        for resolver in self.resolvers.get_resolvers_iter()
         {
             let qresp = 
                 if self.resolvers.option_flags.is_no_parallel() == true
@@ -997,3 +997,35 @@ fn reverse_lookup_a()
     println!("{}", res);
 }
 
+
+#[test]
+fn truncation_test_1()
+{
+    use std::time::Instant;
+
+    let test = QDnsName::from("truncate-zyxw11.go.dnscheck.tools");
+
+    let mut query_setup = QuerySetup::default();
+    query_setup.set_measure_time(true);
+
+
+    let resolvers = CACHE.clone_resolve_list().unwrap();
+    let mut res = 
+        QDns::make_empty(resolvers, 1, query_setup);
+
+    res.add_request(QType::TXT, test);
+    
+    
+    let now = Instant::now();
+    let res = res.query();
+    
+
+    let elapsed = now.elapsed();
+    println!("Elapsed: {:.2?}", elapsed);
+
+  /*  assert_eq!(res.is_ok(), true, "{}", res.err().unwrap());
+    
+    let res = res.unwrap();*/
+    println!("{}", res);
+}
+
